package com.qmth.distributed.print.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.distributed.print.business.enums.TemplateTypeEnum;
import com.qmth.distributed.print.business.mapper.BasicPrintConfigMapper;
import com.qmth.distributed.print.business.service.BasicExamService;
import com.qmth.distributed.print.business.service.BasicPrintConfigService;
import com.qmth.distributed.print.business.service.BasicTemplateOrgService;
import com.qmth.teachcloud.common.bean.dto.DataPermissionRule;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.BasicPrintConfig;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.service.BasicRoleDataPermissionService;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * <p>
 * 印品配置 服务实现类
 * </p>
 *
 * @author xf
 * @since 2022-02-10
 */
@Service
public class BasicPrintConfigServiceImpl extends ServiceImpl<BasicPrintConfigMapper, BasicPrintConfig> implements BasicPrintConfigService {

    @Resource
    private BasicExamService basicExamService;

    @Resource
    BasicTemplateOrgService basicTemplateOrgService;

    @Resource
    BasicPrintConfigService basicPrintConfigService;

    @Resource
    private BasicRoleDataPermissionService basicRoleDataPermissionService;

    @Override
    public IPage<BasicPrintConfig> listPage(Long examId, Boolean enable, Long createStartTime, Long createEndTime, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser requestUser = (SysUser) ServletUtil.getRequestUser();
        DataPermissionRule dpr = basicRoleDataPermissionService.findDataPermission(schoolId, requestUser.getId(), ServletUtil.getRequest().getServletPath());

        Page<BasicPrintConfig> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.listPage(page, schoolId, examId, enable, createStartTime, createEndTime, dpr);
    }

    @Override
    public boolean enable(Long id, Boolean enable) {
        UpdateWrapper<BasicPrintConfig> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicPrintConfig::getEnable, enable).eq(BasicPrintConfig::getId, id);
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean savePrintConfig(BasicPrintConfig basicPrintConfig) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        basicPrintConfig.setSchoolId(schoolId);
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        boolean isSuccess;

        // 新增
        if (Objects.isNull(basicPrintConfig.getId())) {
            basicPrintConfig.setId(SystemConstant.getDbUuid());
            basicPrintConfig.setOrgId(sysUser.getOrgId());
            basicPrintConfig.setCreateId(sysUser.getId());
            basicPrintConfig.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(basicPrintConfig);
        }
        // 修改
        else {
            basicPrintConfig.setUpdateId(sysUser.getId());
            basicPrintConfig.setUpdateTime(System.currentTimeMillis());

            //删除权限
            basicTemplateOrgService.removeByRuleId(basicPrintConfig.getId());
            isSuccess = this.updateById(basicPrintConfig);
        }
        return isSuccess;
    }

    @Override
    public BasicPrintConfig getByExamIdAndCourseCode(Long examId, String courseCode) {
        BasicExam basicExam = basicExamService.getById(examId);

        BasicPrintConfig printConfig = this.baseMapper.getBySchoolIdAndExamId(basicExam.getSchoolId(), examId);
        if (printConfig == null) {
            throw ExceptionResultEnum.ERROR.exception("考试[" + basicExam.getName() + "]未配置印品");
        }
        if (!printConfig.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("考试[" + basicExam.getName() + "]印品配置已禁用");
        }
        return printConfig;
    }

    @Override
    public BasicPrintConfig getByExamIdAndOrgId(Long examId, Long orgId) {
        BasicExam basicExam = basicExamService.getById(examId);
        BasicPrintConfig printConfig = this.baseMapper.getBySchoolIdAndExamId(basicExam.getSchoolId(), examId);
        if (printConfig == null) {
            throw ExceptionResultEnum.ERROR.exception("考试[" + basicExam.getName() + "]未设置印品");
        }
        if (!printConfig.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("考试[" + basicExam.getName() + "]印品配置已禁用");
        }
        return printConfig;
    }

    @Override
    public List<String> listOrgIdsByExamId(Long id, Long examId) {
        QueryWrapper<BasicPrintConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicPrintConfig::getExamId, examId);
        if (id != null) {
            queryWrapper.lambda().ne(BasicPrintConfig::getId, id);
        }
        List<BasicPrintConfig> basicPrintConfigs = this.list(queryWrapper);
        List<String> orgIds = new ArrayList<>();
        for (BasicPrintConfig basicPrintConfig : basicPrintConfigs) {
            List<SysOrg> sysOrgList = basicTemplateOrgService.listByTypeAndTemplateId(TemplateTypeEnum.PRINT_CONFIG.name(), basicPrintConfig.getId());
            for (SysOrg sysOrg : sysOrgList) {
                orgIds.add(String.valueOf(sysOrg.getId()));
            }
        }
        return orgIds;
    }

    @Transactional
    @Override
    public boolean delete(List<Long> idList) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<BasicPrintConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicPrintConfig::getSchoolId, schoolId);
        List<BasicPrintConfig> basicPrintConfigs = basicPrintConfigService.list(queryWrapper);
        List<Long> templateIds = new ArrayList<>();
        if (!basicPrintConfigs.isEmpty()) {
            for (BasicPrintConfig basicPrintConfig : basicPrintConfigs) {
                Set<Long> variableLongs = JSON.parseArray(basicPrintConfig.getVariableContent(), JSONObject.class).stream().map(m -> m.getLong("templateId")).collect(Collectors.toSet());
                if (!variableLongs.isEmpty()) {
                    templateIds.addAll(variableLongs);
                }

                Set<Long> ordinaryLongs = JSON.parseArray(basicPrintConfig.getOrdinaryContent(), JSONObject.class).stream().map(m -> m.getLong("templateId")).collect(Collectors.toSet());
                if (!ordinaryLongs.isEmpty()) {
                    templateIds.addAll(ordinaryLongs);
                }
            }

        }

        for (Long aLong : idList) {
            if (!templateIds.isEmpty() && templateIds.contains(aLong)) {
                throw ExceptionResultEnum.ERROR.exception("印品已经考试绑定，不能删除");
            }
            this.removeById(aLong);
        }
        return true;
    }

    @Override
    public List<BasicPrintConfig> listBySchoolId(Long schoolId) {
        QueryWrapper<BasicPrintConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicPrintConfig::getSchoolId, schoolId);
        return this.list(queryWrapper);
    }
}
