package com.qmth.teachcloud.task.job.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.boot.redis.uid.RedisMachineService;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.enums.PrintPlanStatusEnum;
import com.qmth.distributed.print.business.service.BasicMessageService;
import com.qmth.distributed.print.business.service.ExamPrintPlanService;
import com.qmth.distributed.print.business.service.ExamTaskService;
import com.qmth.distributed.print.business.service.PrintCommonService;
import com.qmth.teachcloud.common.bean.dto.MqDto;
import com.qmth.teachcloud.common.bean.dto.SchoolDto;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.service.BasicSchoolService;
import com.qmth.teachcloud.common.util.RedisUtil;
import com.qmth.teachcloud.task.job.service.JobService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @Description: job service impl
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/31
 */
@Service
public class JobServiceImpl implements JobService {
    private final static Logger log = LoggerFactory.getLogger(JobServiceImpl.class);

    @Resource
    PrintCommonService printCommonService;

    @Resource
    ExamTaskService examTaskService;

    @Resource
    BasicMessageService basicMessageService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    RedisMachineService redisMachineService;

    /**
     * 重新生成pdf失败任务
     *
     * @throws IOException
     */
    @Override
    public void resetCreatePdfTask() throws IOException {
        printCommonService.resetCreatePdfTask();
    }

    @Override
    public void sendSmsExpireTask() {
        examTaskService.sendSmsExpireTask();
    }

    @Override
    public void sendSmsOverdueTask() {
        examTaskService.sendSmsOverdueTask();
    }

    @Override
    public void resendSmsTask() {
        basicMessageService.resendSmsTask();
    }

    @Override
    public void machineHeart() {
        redisUtil.set(SystemConstant.TASK_MACHINE_ID + redisMachineService.getMachineId(), redisMachineService.getMachineId(), 30, TimeUnit.SECONDS);
    }

    @Override
    public void updatePrintPlanStatus() {
        List<SchoolDto> schoolDtos = basicSchoolService.listSchool();
        for (SchoolDto schoolDto : schoolDtos) {
            QueryWrapper<ExamPrintPlan> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamPrintPlan::getSchoolId, schoolDto.getId())
                    .ne(ExamPrintPlan::getStatus, PrintPlanStatusEnum.END)
                    .le(ExamPrintPlan::getExamEndTime, System.currentTimeMillis());
            List<ExamPrintPlan> examPrintPlanList = examPrintPlanService.list(queryWrapper);

            List<Long> ids = examPrintPlanList.stream().map(ExamPrintPlan::getId).collect(Collectors.toList());

            UpdateWrapper<ExamPrintPlan> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(ExamPrintPlan::getStatus, PrintPlanStatusEnum.END)
                    .in(ExamPrintPlan::getId, ids);
            examPrintPlanService.update(updateWrapper);
        }
    }

    /**
     * 组装job
     *
     * @param redisKey
     */
    @Override
    public void assembleJob(String redisKey) {
        Long size = redisUtil.getHashSize(redisKey);
        if (Objects.nonNull(size) && size.longValue() > 0) {
            log.info("redisKey:{}缓冲区的消息数为:{}", redisKey, size);
            Map map = redisUtil.getHashEntries(redisKey);
            map.forEach((k, v) -> {
                MqDto mqDto = (MqDto) v;
                redisUtil.sendMessage(mqDto.getTopic(), mqDto);
            });
        }
    }
}
